from setuptools import setup


setup(
  name='pymonet',
  packages=['pymonet'],
  version='0.5.0',
  description='High abstract python library for functional programming. Contains algebraic data structures known from Haskell or Scala.',
  author='Przemyslaw Jan Pietrzak',
  author_email='przemyslaw.jan.pietrzak@gmail.com',
  url='https://github.com/przemyslawjanpietrzak/pyMonet',
  download_url='https://github.com/przemyslawjanpietrzak/pyMonet/archive/0.5.0.tar.gz',
  keywords=['functional-programming', 'monads', 'functors', 'semigroups'],
  classifiers=[],
)
